<?php

/**
 * This file is dedicated to all functions used to manage the
 * default Mantis functions. 
 *
 * Created: 2007-05-26
 * Last update: 2007-05-28
 *
 * @link http://deboutv.free.fr/mantis/
 * @author Vincent DEBOUT <deboutv@free.fr>
 * @package PluginManager
 * @version 0.1.0
 */

/**
 * This function is used by the install/upgrade script to overwrite an
 * existing function.
 * This function returns '' if the function has been overwritten.
 * If the function has already been overwritten, this function
 * returns the list of the plugins using the overwritten function. The
 * first plugin of the list is the plugin that actually handle the
 * overwrite. Installer must check if it can manage all the requirements
 * of all plugins in the list before retry another overwrite. If it is
 * possible, the installer must recall this function and pass the list
 * of plugins as $p_plugins.
 * If the overwrite fails, this function returns ****ERROR****.
 * 
 * @param string $p_function_name The name of the function to overwrite.
 * @param string $p_file The name of the file where the function is located.
 * The file _must_ be relative to the Mantis root directory. 
 * @param string $p_overwrite_function The name of the new function to
 * execute.
 * @param string $p_plugin The name of the plugin that overwrites the
 * function.
 * @param string $p_plugins The list of the plugin that have already
 * overwrite the function.
 * @return string
 */
function pm_function_add_overwrite( $p_function_name, $p_file, $p_overwrite_function, $p_plugin, $p_plugins = '' ) {
    $t_query = 'SELECT plugins FROM ' . config_get( 'db_table_prefix' ) . '_plugins_function_overwrite_table WHERE function=\'' . db_prepare_string( $p_function_name ) . '\'';
    $t_result = db_query( $t_query );
    if ( db_num_rows( $t_result ) == 1 && $p_plugins == '' ) {
        $t_row = db_fetch_array( $t_result );
        return $t_row['plugins'];
    } else if ( db_num_rows( $t_result ) == 1 ) {
        $t_query = 'UPDATE ' . config_get( 'db_table_prefix' ) . '_plugins_function_overwrite_table SET new_function=\'' . db_prepare_string( $p_overwrite_function ) . '\', plugins=\'' . db_prepare_string( $p_plugin . ',' . $p_plugins ) . '\' WHERE function=\'' . db_prepare_string( $p_function_name ) . '\' AND plugins=\'' . db_prepare_string( $p_plugins ) . '\'';
        db_query( $t_query );
        return '';
    }
    $t_query = 'INSERT INTO ' . config_get( 'db_table_prefix' ) . '_plugins_function_overwrite_table ( plugin, function, new_function, plugins ) VALUES ( \'' . db_prepare_string( $p_plugin ) . '\', \'' . db_prepare_string( $p_function_name ) . '\', \'' . db_prepare_string( $p_overwrite_function ) . '\', \'' . db_prepare_string( $p_plugin ) . '\' )';
    $t_result = db_query( $t_query );
    if ( $t_result ) {
        if ( pm_backup_file( $p_file ) && is_writeable( dirname( dirname( dirname( dirname( __FILE__ ) ) ) ) . DIRECTORY_SEPARATOR . $p_file ) ) {
            $t_file = fopen( dirname( dirname( dirname( dirname( __FILE__ ) ) ) ) . DIRECTORY_SEPARATOR . $p_file, 'r' );
            $t_file_content = fread( $t_file, filesize( dirname( dirname( dirname( dirname( __FILE__ ) ) ) ) . DIRECTORY_SEPARATOR . $p_file ) );
            fclose( $t_file );
            $t_new_file_content = preg_replace_callback( '/function[ \t\n]+(' . $p_function_name . ')[ \t\n]*\((.*)\)[ \t\n]*{/', 'plugins_pluginmanager_function_callback_replace', $t_file_content );
            if ( $t_new_file_content != $t_file_content ) {
                $t_file = fopen( dirname( dirname( dirname( dirname( __FILE__ ) ) ) ) . DIRECTORY_SEPARATOR . $p_file, 'w' );
                fwrite( $t_file, $t_new_file_content );
                fclose( $t_file );
                return '';
            }
            pm_backup_remove_file( $p_file );
        }
        $t_query = 'DELETE FROM ' . config_get( 'db_table_prefix' ) . '_plugins_function_overwrite_table WHERE function=\'' . db_prepare_string( $p_function_name ) . '\'';
        db_query( $t_query );
    }
    return '****ERROR****';
}

/**
 * This function is used by the install/upgrade script to remove an
 * overwritten function.
 * 
 * @param string $p_function_name The name of the function to overwrite.
 * @param string $p_plugin The name of the plugin that overwrites the
 * function.
 * @return string
 */
function pm_function_remove_overwrite( $p_function_name, $p_plugin ) {
    $t_query = 'DELETE FROM ' . config_get( 'db_table_prefix' ) . '_plugins_function_overwrite_table WHERE function=\'' . db_prepare_string( $p_function_name ) . '\'';
    db_query( $t_query );
    return '****ERROR****';
}

/** 
 * @access private
 */
function plugins_pluginmanager_function_overwrite( $p_function_name, $p_arguments = array() ) {
    $t_function_name = 'pm_old_' . $p_function_name;
    $t_query = 'SELECT new_function FROM ' . config_get( 'db_table_prefix' ) . '_plugins_function_overwrite_table WHERE function=\'' . db_prepare_string( $p_function_name ) . '\'';
    $t_result = db_query( $t_query );
    if ( db_num_rows( $t_result ) == 1 ) {
        $t_row = db_fetch_array( $t_result );
        /**
        * Prevent infinite loop if the new_function is the same as
        * the old one
        */
        if ( $t_row['new_function'] != $p_function_name ) {
            $t_function_name = $t_row['new_function'];
        }
    }
    return call_user_func_array ( $t_function_name, $p_arguments );
}

/** 
 * @access private
 */
function plugins_pluginmanager_function_callback_replace( $p_matches ) {
    $t_result = $p_matches[0];
    $t_result .= "\n\t" . 'plugins_pluginmanager_function_overwrite( \'' . $p_matches[1] . '\', array( ';
    $t_first = true;
    foreach( explode( ' ', $p_matches[2] ) as $t_match ) {
        if ( substr( $t_match, 0 ) == '$' ) {
            if ( $t_first ) {
                $t_first = false;
            } else {
                $t_result .= ', ';
            }
            $t_result .= $t_match;
        }
    }
    $t_result .= ' ) );';
    $t_result .= "\n}\n\n";
    $t_result .= str_replace( $p_matches[1], 'pm_old_' . $p_matches[1], $p_matches[0] );
    return $t_result;
}
?>
