<?php

/**
 * This file is dedicated to all plugin functions.
 *
 * Created: 2007-02-12
 * Last update: 2007-05-27
 *
 * @link http://deboutv.free.fr/mantis/
 * @author Vincent DEBOUT <deboutv@free.fr>
 * @package PluginManager
 * @version 0.1.0
 */

/**
 * This function returns the plugin info.
 *
 * @param string $p_plugin The plugin
 * @param bool $p_check_installed If set to true, check also if the
 * plugin is installed.
 * @return array
 */
function pm_get_plugin_info( $p_plugin, $p_check_installed = true ) {
    $t_directory = dirname( dirname( dirname( __FILE__ ) ) );
    if ( $p_check_installed && !pm_plugin_is_installed( $p_plugin ) ) {
        return false;
    }
    if ( is_dir( $t_directory . DIRECTORY_SEPARATOR . $p_plugin ) ) {
        $t_file = $t_directory . DIRECTORY_SEPARATOR . $p_plugin . DIRECTORY_SEPARATOR . 'info.php';
        $t_function = $p_plugin . '_get_info';
        if ( file_exists( $t_file ) && !function_exists( $t_function ) ) {
            include( $t_file );
            if ( function_exists( $t_function ) ) {
                return $t_function();
            } else {
                return false;
            }
        } elseif( function_exists( $t_function ) ) {
            return $t_function();
        }
    }
    return false;
}

/**
 * This function returns the list of installed plugins.
 *
 * @return array
 */
function pm_get_installed_plugin_list() {
    $t_query = 'SELECT plugin FROM ' . config_get( 'db_table_prefix' ) . '_plugins_pm_list_table WHERE installed=1';
    $t_result = db_query( $t_query );
    $t_res = array();
    while( $t_row = db_fetch_array( $t_result ) ) {
        $t_res[] = $t_row['plugin'];
    }
    return $t_res;
}

/** 
 * This function returns true if the plugin is installed
 * on the bugtracker and returns false in other cases.
 *
 * @param string $p_plugin The plugin id (name of the directory).
 * @return bool
 */
function pm_plugin_is_installed( $p_plugin ) {
    $t_query = 'SHOW TABLES';
    $t_result = db_query( $t_query );
    $t_found = false;
    $t_table = config_get( 'db_table_prefix' ) . '_plugins_pm_list_table';
    while( $t_row = db_fetch_array( $t_result ) ) {
        if ( $t_row[0] == $t_table ) {
            $t_found = true;
            break;
        }
    }
    if ( !$t_found ) {
        return false;
    }
    $t_query = 'SELECT plugin FROM ' . $t_table . ' WHERE plugin=\'' . db_prepare_string( $p_plugin ) . '\'';
    $t_result = db_query( $t_query );
    if ( db_num_rows( $t_result ) != 1 ) {
        return false;
    }
    return true;
}

/**
 * @access private
 */
function plugins_pluginmanager_get_plugin_list( $p_check = true, $p_include = false ) {
    $t_directory = dirname( dirname( dirname( __FILE__ ) ) );
    $t_dir = opendir( $t_directory );
    
    if ( !$p_include ) {
        $t_access_level = user_get_access_level( auth_get_current_user_id(), helper_get_current_project() );
    } else {
        $t_access_level = VIEWER;
    }
    
    $t_plugin_list = array();
    $t_installed_plugin = pm_get_installed_plugin_list();
    while( ( $t_file = readdir( $t_dir ) ) !== false ) {
        if ( is_dir( $t_directory . DIRECTORY_SEPARATOR . $t_file ) && $t_file != '.' && $t_file != '..' ) {
            $t_plugin = array();
            $t_plugin['id'] = $t_file;
            $t_file = $t_directory . DIRECTORY_SEPARATOR . $t_file . DIRECTORY_SEPARATOR . 'info.php';
            $t_function = $t_plugin['id'] . '_get_info';
            if ( file_exists( $t_file ) && !function_exists( $t_function ) ) {
                include( $t_file );
                if ( function_exists( $t_function ) ) {
                    $t_plugin_info = $t_function();
                    $t_plugin['name'] = $t_plugin_info['name'];
                    if ( isset( $t_plugin_info['url'] ) ) {
                        $t_plugin['url'] = $t_plugin_info['url'];
                    }
                    $t_plugin['version'] = $t_plugin_info['version'];
                    if ( $p_check && isset( $t_plugin_info['check'] ) && config_get( 'plugins_pluginmanager_check_version', PLUGINS_PLUGINMANAGER_CHECK_VERSION_DEFAULT ) == ON ) {
                        $t_url = @fopen( $t_plugin_info['check'], 'r' );
                        if ( $t_url ) {
                            $t_version = fscanf( $t_url, "%s\n" );
                            $t_plugin['check'] = $t_version[0];
                            fclose( $t_url );
                        } else {
                            $t_plugin['check'] = '-';
                        }
                    } else {
                        $t_plugin['check'] = '-';
                    }
                    $t_threshold = config_get( 'plugins_pluginmanager_' . $t_plugin['id'] . '_threshold', PLUGINS_PLUGINMANAGER_CONFIGURE_THRESHOLD_DEFAULT );
                    if ( $t_access_level >= $t_threshold && in_array( $t_plugin['id'], $t_installed_plugin ) ) {
                        $t_plugin['has_access_level'] = true;
                    } else {
                        $t_plugin['has_access_level'] = false;
                    }
                    $t_file = str_replace( 'info.php', 'install_inc.php', $t_file );
                    if ( file_exists( $t_file ) ) {
                        $t_plugin['can_be_upgraded_uninstalled'] = true;
                    } else {
                        $t_plugin['can_be_upgraded_uninstalled'] = false;
                    }
                    $t_file = str_replace( 'install_inc.php', 'package_inc.php', $t_file );
                    if ( file_exists( $t_file ) && $t_plugin['can_be_upgraded_uninstalled'] ) {
                        $t_plugin['can_be_packaged'] = true;
                    } else {
                        $t_plugin['can_be_packaged'] = false;
                    }
                    $t_plugin_list[] = $t_plugin;
                }
            } elseif( function_exists( $t_function ) ) {
                $t_plugin_info = $t_function();
                $t_plugin['name'] = $t_plugin_info['name'];
                if ( isset( $t_plugin_info['url'] ) ) {
                    $t_plugin['url'] = $t_plugin_info['url'];
                }
                $t_plugin['version'] = $t_plugin_info['version'];
                if ( $p_check && isset( $t_plugin_info['check'] ) && config_get( 'plugins_pluginmanager_check_version', PLUGINS_PLUGINMANAGER_CHECK_VERSION_DEFAULT ) == ON ) {
                    $t_url = @fopen( $t_plugin_info['check'], 'r' );
                    if ( $t_url ) {
                        $t_version = fscanf( $t_url, '%s\n' );
                        $t_plugin['check'] = $t_version[0];
                        fclose( $t_url );
                    } else {
                        $t_plugin['check'] = '-';
                    }
                } else {
                    $t_plugin['check'] = '-';
                }
                $t_threshold = config_get( 'plugins_pluginmanager_' . $t_plugin['id'] . '_threshold', PLUGINS_PLUGINMANAGER_CONFIGURE_THRESHOLD_DEFAULT );
                if ( $t_access_level >= $t_threshold && in_array( $t_plugin['id'], $t_installed_plugin ) ) {
                    $t_plugin['has_access_level'] = true;
                } else {
                    $t_plugin['has_access_level'] = false;
                }
                $t_file = str_replace( 'info.php', 'install_inc.php', $t_file );
                if ( file_exists( $t_file ) ) {
                    $t_plugin['can_be_upgraded_uninstalled'] = true;
                } else {
                    $t_plugin['can_be_upgraded_uninstalled'] = false;
                }
                $t_file = str_replace( 'install_inc.php', 'package_inc.php', $t_file );
                if ( file_exists( $t_file ) && $t_plugin['can_be_upgraded_uninstalled'] ) {
                    $t_plugin['can_be_packaged'] = true;
                } else {
                    $t_plugin['can_be_packaged'] = false;
                }
                $t_plugin_list[] = $t_plugin;
            } elseif ( file_exists( $t_directory . DIRECTORY_SEPARATOR . $t_plugin['id'] . DIRECTORY_SEPARATOR . 'index.php' ) ) {
                $t_plugin['name'] = $t_plugin['id'];
                $t_plugin['version'] = '--';
                $t_plugin['check'] = '--';
                if ( $t_access_level >= ADMINISTRATOR && in_array( $t_plugin['id'], $t_installed_plugin ) ) {
                    $t_plugin['has_access_level'] = true;
                } else {
                    $t_plugin['has_access_level'] = false;
                }
                $t_plugin['can_be_upgraded_uninstalled'] = false;
                $t_plugin['can_be_packaged'] = false;
                $t_plugin_list[] = $t_plugin;
            }
        }
    }

    asort( $t_plugin_list );
    for( $i=0; $i<count( $t_plugin_list ); $i++ ) {
        if ( !in_array( $t_plugin_list[$i]['id'], $t_installed_plugin ) ) {
            $t_plugin_list[$i]['can_be_upgraded_uninstalled'] = false;
            $t_plugin_list[$i]['must_be_installed'] = true;
        }
    }
    return $t_plugin_list;
}

/**
 * @access private
 */
function plugins_pluginmanager_get_plugin_access_level( $p_plugin ) {
    return config_get( 'plugins_pluginmanager_' . $p_plugin . '_threshold', PLUGINS_PLUGINMANAGER_CONFIGURE_THRESHOLD_DEFAULT );
}

/** 
 * @access private
 */
function plugins_pluginmanager_print_test_result( $p_result, $p_message='' ) {
    global $g_failed;
    
    echo '<td class="center" ';
    if ( PLUGINS_PLUGINMANAGER_FAIL == $p_result ) {
        $g_failed = true;
        echo 'bgcolor="red">FAIL';
        if ( '' != $p_message ) {
            echo '<br />' . $p_message;
        }   
    } else if ( PLUGINS_PLUGINMANAGER_WARNING == $p_result ) {
        echo 'bgcolor="pink">WARNING';
        if ( '' != $p_message ) {
            echo '<br />' . $p_message;
        }
    } else if ( PLUGINS_PLUGINMANAGER_OK == $p_result ) {
        echo 'bgcolor="green">';
        if ( $p_message == '' ) {
            echo 'OK';
        } else {
            echo $p_message;
        }
    }
    
    echo '</td>';
}

/** 
 * @access private
 */
function plugins_pluginmanager_print_test( $p_test_description, $p_result, $p_message='' ) {
    echo "\n" . '<tr ' . helper_alternate_class() . '><td>' . $p_test_description . '</td>';
    plugins_pluginmanager_print_test_result( $p_result, $p_message );
    echo "</tr>\n";
}

/** 
 * @access private
 */
function plugins_pluginmanager_install_file( &$p_failed, &$p_msg, $p_file, $p_content ) {
    global $g_installed_files;
    
    if ( $p_failed == PLUGINS_PLUGINMANAGER_OK && file_exists( $p_file ) ) {
        if ( !pm_backup_file( $p_file ) ) {
            $p_failed = PLUGINS_PLUGINMANAGER_FAIL;
            $p_msg = 'could not backup file ' . $p_file;
        } else {
            $g_installed_files['rename'][] = $p_file;
        }
    }
    if ( $p_failed == PLUGINS_PLUGINMANAGER_OK ) {
        $t_directory = '';
        $t_dirs = explode( DIRECTORY_SEPARATOR, $p_file );
        for( $i=0; $i<count( $t_dirs ) - 1; $i++ ) {
            $t_directory .= $t_dirs[$i] . DIRECTORY_SEPARATOR;
            if ( !is_dir( $t_directory ) ) {
                if ( $p_failed == PLUGINS_PLUGINMANAGER_OK && !@mkdir( $t_directory ) ) {
                    $p_failed = PLUGINS_PLUGINMANAGER_FAIL;
                    $p_msg = 'could not create directory ' . $t_directory;
                }
            }
        }
        if ( $p_failed == PLUGINS_PLUGINMANAGER_OK ) {
            $t_file = @fopen( $p_file, 'w' );
            if ( $t_file ) {
                fwrite( $t_file, base64_decode( $p_content ) );
                fclose( $t_file );
                $g_installed_files['new'][] = $p_file;
            } else {
                $p_failed = PLUGINS_PLUGINMANAGER_FAIL;
                $p_msg = 'could not write file ' . $p_file;
            }
        }
    }
}

/** 
 * @access private
 */
function plugins_pluginmanager_restore( &$p_failed, &$p_msg ) {
    global $g_installed_files;

    $p_msg = '';
    if ( isset( $g_installed_files['new'] ) ) {
        foreach( $g_installed_files['new'] as $t_file ) {
            if ( !@unlink( $t_file ) ) {
                $p_failed = PLUGINS_PLUGINMANAGER_FAIL;
                $p_msg = 'unable to restore';
            }
        }
    }
    if ( isset( $g_installed_files['rename'] ) ) {
        foreach( $g_installed_files['rename'] as $t_file ) {
            if ( !pm_backup_restore_file( $t_file ) ) {
                $p_failed = PLUGINS_PLUGINMANAGER_FAIL;
                $p_msg = 'unable to restore';
            }
        }
    }
}

/** 
 * @access private
 */
function plugins_pluginmanager_default_description_plugin( $p_step ) {
    return '';
}

/** 
 * @access private
 */
function plugins_pluginmanager_get_plugin_dependencies( $p_plugin ) {
    $t_query = 'SELECT plugin FROM ' . config_get( 'db_table_prefix' ) . '_plugins_pm_dependencies_table WHERE dependencies LIKE \'%,' . db_prepare_string( $p_plugin ) . ',%\'';
    $t_result = db_query( $t_query );
    while( $t_row = db_fetch_array( $t_result ) ) {
        $t_res[] = $t_row['plugin'];
    }
    if ( isset( $t_res ) ) {
        return $t_res;
    } else {
        return false;
    }
}

?>
